<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use \App\Models\MyUser; //Chargement du model

class UserController extends Controller
{
    /**
     * Show the signin page
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function signin(Request $request)
    {
        return view('signin', ['message' => $request->session()->get('message') ?? null]);
    }

    public function signup(Request $request)
    {
        return view('signup', ['message' => $request->session()->get('message') ?? null]);
    }

    public function formpassword(Request $request)
    {
        return view('formpassword', ['message' => $request->session()->get('message') ?? null]);
    }

    public function signout(Request $request)
    {
        $request->session()->flush();
        return redirect()->route('signin');
    }

    public function account(Request $request)
    {
        return view(
            'account',
            ['message' => $request->session()->get('message') ?? null],
            ['user' => $request->session()->get('user')]
        );
    }

    public function authenticated(Request $request)
    {
        /******************************************************************************
         * Initialisation.
         */
        $request->session()->forget('message');

        /******************************************************************************
         * Traitement des données de la requête
         */
        //Utilisation de la méthode input() de $request pour récupérer les variables
        $login = $request->input('login');
        $password = $request->input('password');

        // 2. On vérifie que les données attendues existent
        if (empty($login) || empty($password)) {
            $request->session()->put('message', 'Some POST data are missing.');
            return redirect()->route('signin');
            exit();
        }

        // 3. On sécurise les données reçues
        $login = htmlspecialchars($login);
        $password = htmlspecialchars($password);

        /******************************************************************************
         * Authentification
         */

        // 1. On crée l'utilisateur avec les identifiants passés en POST
        $user = new MyUser($login, $password);

        // 2. On vérifie qu'il existe dans la BDD
        try {
            if (!$user->exists()) {
                $request->session()->put('message', 'Wrong login/password');
                return redirect()->route('signin');
                exit();
            }
        } catch (\PDOException $e) {
            // Si erreur lors de la création de l'objet PDO
            // (déclenchée par MyPDO::pdo())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('signin');
            exit();
        } catch (\Exception $e) {
            // Si erreur durant l'exécution de la requête
            // (déclenchée par le throw de $user->exists())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('signin');
            exit();
        }

        // 3. On sauvegarde le login dans la session
        $request->session()->put('user', $login);

        // 4. On sollicite une redirection vers la page du compte
        return redirect()->route('account');
        exit();
    }

    public function adduser(Request $request)
    {
        /******************************************************************************
         * Initialisation.
         */
        unset($_SESSION['message']);

        /******************************************************************************
         * Traitement des données de la requête
         */
        //Utilisation de la méthode input() de $request pour récupérer les variables
        $login = $request->input('login');
        $password = $request->input('password');
        $confirm = $request->input("confirm");

        // 2. On vérifie que les données attendues existent
        if (empty($login) || empty($password) || empty($confirm)) {
            $request->session()->put('message', "Some POST data are missing.");
            return redirect()->route('signup');
            exit();
        }

        // 3. On sécurise les données reçues
        $login = htmlspecialchars($login);
        $password = htmlspecialchars($password);
        $confirm = htmlspecialchars($confirm);

        // 4. On vérifie que les deux mots de passe correspondent
        if ($password !== $confirm) {
            $request->session()->put('message', "The thow pass are differents");
            return redirect()->route('signup');
            exit();
        }

        /******************************************************************************
         * Ajout de l'utilisateur
         */

        // 1. On crée l'utilisateur avec les identifiants passés en POST
        $user = new MyUser($login, $password);

        // 2. On crée l'utilisateur dans la BDD
        try {
            $user->create();
        } catch (\PDOException $e) {
            // Si erreur lors de la création de l'objet PDO
            // (déclenchée par MyPDO::pdo())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('signup');
            exit();
        } catch (\Exception $e) {
            // Si erreur durant l'exécution de la requête
            // (déclenchée par le throw de $user->create())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('signup');
            exit();
        }

        // 3. On indique que le compte a bien été créé
        $request->session()->put('message', "Account created! Now signin");

        // 4. On sollicite une redirection vers la page d'accueil
        return redirect()->route('signin');
        exit();
    }

    public function changepassword(Request $request)
    {

        /******************************************************************************
         * Initialisation.
         */
        $request->session()->forget('message');

        /******************************************************************************
         * Vérification de la session
         */

        // 2. On récupère le login dans une variable
        $login = $request->session()->get('user');

        /******************************************************************************
         * Traitement des données de la requête
         */
        //Utilisation de la méthode input() de $request pour récupérer les variables
        $newpassword = $request->input('newpassword');
        $confirmpassword = $request->input('confirmpassword');

        // 2. On vérifie que les données attendues existent
        if (empty($newpassword) || empty($confirmpassword)) {
            $request->session()->put('message', "Some POST data are missing.");
            return redirect()->route('formpassword');
            exit();
        }

        // 3. On sécurise les données reçues
        $newpassword     = htmlspecialchars($newpassword);
        $confirmpassword = htmlspecialchars($confirmpassword);

        // 4. On s'assure que les 2 mots de passes sont identiques
        if ($newpassword != $confirmpassword) {
            $request->session()->put('message', "Error: passwords are different.");
            return redirect()->route('formpassword');
            exit();
        }

        /******************************************************************************
         * Changement du mot de passe
         */

        // 1. On crée l'utilisateur avec les identifiants passés en POST
        $user = new MyUser($login);

        // 2. On change le mot de passe de l'utilisateur
        try {
            $user->changePassword($newpassword);
        } catch (\PDOException $e) {
            // Si erreur lors de la création de l'objet PDO
            // (déclenchée par MyPDO::pdo())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('formpassword');
            exit();
        } catch (\Exception $e) {
            // Si erreur durant l'exécution de la requête
            // (déclenchée par le throw de $user->changePassword())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('formpassword');
            exit();
        }

        // 3. On indique que le mot de passe a bien été modifié
        $request->session()->put('message', "Password successfully updated.");

        // 4. On sollicite une redirection vers la page du compte
        return redirect()->route('account');
        exit();
    }

    public function deleteuser(Request $request)
    {
        /******************************************************************************
         * Initialisation.
         */
        $request->session()->forget('message');

        /******************************************************************************
         * Vérification de la session
         */

        // 2. On récupère le login dans une variable
        $login = $request->session()->get('user');

        /******************************************************************************
         * Suppression de l'utilisateur
         */

        // 1. On crée l'utilisateur avec les identifiants passés en POST
        $user = new MyUser($login);

        // 2. On détruit l'utilisateur dans la BDD
        try {
            $user->delete();
        } catch (\PDOException $e) {
            // Si erreur lors de la création de l'objet PDO
            // (déclenchée par MyPDO::pdo())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('account');
            exit();
        } catch (\Exception $e) {
            // Si erreur durant l'exécution de la requête
            // (déclenchée par le throw de $user->create())
            $request->session()->put('message', $e->getMessage());
            return redirect()->route('account');
            exit();
        }

        // 3. On détruit la session
        $request->session()->flush();

        // 4. On crée une nouvelle session
        $request->session()->regenerate();

        // 5. On indique que le compte a bien été supprimé
        $request->session()->put('message', "Account successfully deleted.");

        // 4. On sollicite une redirection vers la page d'authentification
        return redirect()->route('signin');
        exit();
    }
}
