<?php

namespace App\Http\Controllers;

use App\Models\Bird;
use Illuminate\Http\Request;

class BirdController extends Controller
{
    /**
	 * My profil
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function formBird( Request $request )
	{
		return view('formBird')
		->with('user',$request->session()->get('user'))
		->with('message',$request->session()->get('message'));
    }
    
    /**
	 * Add a new bird
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function addbird( Request $request )
	{
        if ( !$request->filled(['date_observ','lieux_observ','espece','nom_science','desc']) )
        return redirect()->route('signup')->with('message','Some POST data are missing.');

        $bird = new Bird();
        $bird->date_observ = $request->date_observ;
        $bird->lieux_observ =  $request->lieux_observ;
        $bird->espece = $request->espece;
        $bird->nom_science = $request->nom_science;
        $bird->desc = $request->desc;

        $user = $request->session()->get('user');
        $bird->user_id = $user->user_id;

        try
        {
            $bird->save();
        }
        catch (\Illuminate\Database\QueryException $e)
        {
            return redirect()->route('formBird')
                ->with('message','Error: '.$e->getMessage());
        }

        return redirect()->route('formBird')->with('message','Bird created! Check in birdList.');
    }
    
    /**
	 * Show the birdsList page
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function birdList(Request $request)
	{
        $allBirds = Bird::all();
        return view('birdList')
            ->with('birds',$allBirds)
            ->with('message',$request->session()->get('message'));
    }

    /**
	 * Delete a bird from birdList page
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function deletebird(Request $request, int $id)
	{
        try
		{
			Bird::where('id',$id)->delete();
		}
		catch (\Illuminate\Database\Eloquent\ModelNotFoundException $e)
		{
			return redirect()->route('account')->with('message','Error during deletion. Try again.');
		}

		return redirect()->route('birdList')->with('message','Bird successfully deleted.');
    }

    /**
	 * Show the Bird's page
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function bird( Request $request, int $id )
	{
        return view('bird')
        ->with('bird',Bird::findOrFail($id))
        ->with('user',$request->session()->get('user'))
        ->with('message',$request->session()->get('message'));
    }

    /**
	 * Show the Bird from place's page
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function birdfromplace( Request $request )
	{
        return view('birdfromplace')
        ->with('birds',Bird::all())
        ->with('user',$request->session()->get('user'))
        ->with('message',$request->session()->get('message'));
        //Manque de temps pour bien développé cette vue
    }
}
